const nodemon = require('nodemon');
const open = require('open');
const webpack = require('webpack');
const appConfig = require('../common/appConfig');
const getEnvironmentInfo = require('../common/extendEnv');
const { getProjectPath, parseArgsToJson } = require('../common/utils');
const clientConfigFactory = require('./webpack.dev.client');
const serverConfigFactory = require('./webpack.dev.server');

const extendEnv = getEnvironmentInfo(true);
const clientConfig = clientConfigFactory(appConfig, extendEnv);
const serverConfig = serverConfigFactory(appConfig, extendEnv);

const PORT = process.env.PORT || parseArgsToJson.port || appConfig.hotSsrDefaultPort;
const hotSsrOpenPath = appConfig.hotSsrOpenPath ? appConfig.hotSsrOpenPath : '';

process.env.BABEL_ENV = 'development';
process.env.NODE_ENV = 'development';

const options = {
  // 配置文件
  script: getProjectPath(appConfig.hotSsrRunPath),
  restartable: 're',
  verbose: true,
  delay: 3000,
  watch: appConfig.hotSsrWatch,
  env: {
    SERVER_PORT: PORT,
    SERVER_RUN: 'NODE',
    ...extendEnv.raw
  }
};

const clientCompiler = webpack(clientConfig);
const serverCompiler = webpack(serverConfig);

let isOpened = false;

function openServer() {
  nodemon(options)
    .on('start', () => {
      // 启动时的事件
      // eslint-disable-next-line no-console
      console.log('Server app has started');
      if (!isOpened) {
        open(`http://localhost:${PORT}${hotSsrOpenPath}`);
        isOpened = true;
      }
    })
    .on('quit', () => {
      // 退出时的事件
      // eslint-disable-next-line no-console
      console.log('Server app has quit');
      process.exit();
    })
    .on('restart', () => {
      // 重启时的事件
      // eslint-disable-next-line no-console
      console.log('Server app has restarted');
    });
}

function run() {
  const compiler = new Promise((resolve, reject) => {
    let clientWatched = false;
    let serverWatched = false;
    let isResolved = false;

    function completeAndResolve() {
      if (clientWatched && serverWatched && !isResolved) {
        isResolved = true;
        return resolve();
      }
      return null;
    }

    function watchingHandler(err, stats) {
      const info = stats.toJson({
        assets: false,
        hash: false,
        colors: false
      });
      if (stats.hasErrors()) {
        // eslint-disable-next-line no-console
        info.errors.forEach(item => console.log(item));
      }

      if (stats.hasWarnings()) {
        info.warnings.forEach(item => {
          if (item && item.message && item.message.indexOf('Conflicting order') > -1) {
            return;
          }
          // eslint-disable-next-line no-console
          console.log(item);
        });
      }
    }

    const clientWatching = clientCompiler.watch(
      {
        ignored: /node_modules/,
        aggregateTimeout: 600
      },
      // eslint-disable-next-line consistent-return
      (err, stats) => {
        if (err) {
          // eslint-disable-next-line no-console
          console.log(err);
          clientWatching.invalidate();
          return reject();
        }
        watchingHandler(err, stats);

        clientWatched = true;
        completeAndResolve();
      }
    );
    const serverWatching = serverCompiler.watch(
      {
        ignored: /node_modules/,
        aggregateTimeout: 600
      },
      // eslint-disable-next-line consistent-return
      (err, stats) => {
        if (err) {
          // eslint-disable-next-line no-console
          console.log(err);
          serverWatching.invalidate();
          return reject();
        }

        watchingHandler(err, stats);

        serverWatched = true;
        completeAndResolve();
      }
    );
  });

  compiler
    .then(() => {
      openServer();
    })
    // eslint-disable-next-line no-console
    .catch(error => console.log(error));
}

run();
